/*
 * This Java source file was generated by the Gradle 'init' task.
 */
package lightcontainer;

import lightcontainer.domains.FFETimer;
import lightcontainer.domains.server.MulticastServerListener;
import lightcontainer.domains.server.UnicastServerListener;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.protocol.rules.reader.*;
import lightcontainer.protocol.rules.writer.*;
import lightcontainer.repository.ClientHandlerRepository;
import lightcontainer.repository.FileFrontEnd;
import lightcontainer.repository.ProtocolRepositoryImpl;
import lightcontainer.repository.StoreProcessorRepository;
import lightcontainer.storage.AppData;
import lightcontainer.storage.JsonAdapter;
import lightcontainer.storage.Repository;

import java.nio.file.Paths;
import java.util.Timer;

public class App {

    public static void main(String[] args) {
        setupVM();
        Repository repositoryStorage = prepareStorage();

        // Create all repository
        ClientHandlerRepository clientRep = new ClientHandlerRepository();
        StoreProcessorRepository storeRep = new StoreProcessorRepository();

        // Initialisation des protocoles
        ProtocolRepository protocolRep = new ProtocolRepositoryImpl();
        initProtocols(repositoryStorage, protocolRep);

        // Initialisation du dispatcher et des servers
        FileFrontEnd ffe = new FileFrontEnd(clientRep, storeRep, protocolRep);
        new UnicastServerListener(ffe, clientRep, protocolRep, repositoryStorage, repositoryStorage.getUnicastPort());
        new MulticastServerListener(ffe, storeRep, protocolRep, repositoryStorage.getMulticastIp(), repositoryStorage.getMulticastPort(), repositoryStorage.getNetworkInterface());

        // S'occupe de distribué les timeout pour les SBE plus connecté et donc de Timeout les clients
        Timer ffeTimer = new Timer();
        ffeTimer.schedule(new FFETimer(storeRep), 50000, 50000);

    }

    private static void initProtocols(Repository repositoryStorage, ProtocolRepository protocolRep) {
        initReadersProtocols(repositoryStorage, protocolRep);
        initWritersProtocols(repositoryStorage, protocolRep);
    }

    private static void initReadersProtocols(Repository repositoryStorage, ProtocolRepository protocolRep) {
        protocolRep.addReader(new HelloRule());
        protocolRep.addReader(new SigninRule(protocolRep));
        protocolRep.addReader(new SignupRule(protocolRep));
        protocolRep.addReader(new SignoutRule());
        protocolRep.addReader(new FilelistRule(protocolRep));
        protocolRep.addReader(new SavefileRule(protocolRep, repositoryStorage.getStoragePath()));
        protocolRep.addReader(new SendOkRule(protocolRep));
        protocolRep.addReader(new SendErrorRule(protocolRep));
        protocolRep.addReader(new GetFileRule(protocolRep));
        protocolRep.addReader(new EraseErrorRule(protocolRep));
        protocolRep.addReader(new EraseOkRule(protocolRep));
        protocolRep.addReader(new RemoveFileRule(protocolRep));
        protocolRep.addReader(new RetrieveErrorRule(protocolRep));
        protocolRep.addReader(new RetrieveOkRule(protocolRep, repositoryStorage.getStoragePath()));
    }

    private static void initWritersProtocols(Repository repositoryStorage, ProtocolRepository protocolRep) {
        protocolRep.addWriter(new SignOkRule());
        protocolRep.addWriter(new SignErrorRule());
        protocolRep.addWriter(new FilesRule());
        protocolRep.addWriter(new SaveFileOkRule(repositoryStorage.getStoragePath()));
        protocolRep.addWriter(new SaveFileErrorRule(repositoryStorage.getStoragePath()));
        protocolRep.addWriter(new SendfileRule(repositoryStorage.getStoragePath()));
        protocolRep.addWriter(new GetFileErrorRule(repositoryStorage.getStoragePath()));
        protocolRep.addWriter(new EraseFileRule());
        protocolRep.addWriter(new RemoveFileErrorRule());
        protocolRep.addWriter(new RemoveFileOkRule());
        protocolRep.addWriter(new GetFileOkRule(repositoryStorage.getStoragePath()));
        protocolRep.addWriter(new RetrieveFileRule());
    }

    private static Repository prepareStorage() {
        AppData appData = AppData.getInstance();
        Repository repository = new Repository(
                Paths.get("src", "main", "resources", "appdata.json").toAbsolutePath().toString(),
                appData, new JsonAdapter()
        );

        repository.load();
        return repository;
    }

    private static void setupVM() {
        System.setProperty("javax.net.ssl.keyStore", "../ffe.labo.swilabus.com.p12");
        System.setProperty("javax.net.ssl.keyStorePassword", "labo2022");
        System.setProperty("https.protocols", "TLSv1.3");
    }

}
