package lightcontainer.domains.server;

import lightcontainer.domains.client.ClientHandler;
import lightcontainer.domains.client.Context;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.interfaces.UnicastCHR;
import lightcontainer.repository.FileFrontEnd;
import lightcontainer.storage.Repository;

import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLSocket;
import java.io.IOException;
import java.net.ServerSocket;

public class UnicastServerListener implements Runnable {
    // Variables
    private ServerSocket server;
    private FileFrontEnd ffe;
    private final UnicastCHR repository;
    private ProtocolRepository protocolRep;
    private final int server_port;
    private boolean server_run;
    private Repository repositoryStorage;

    // Constructor
    public UnicastServerListener(FileFrontEnd ffe, UnicastCHR repository, ProtocolRepository protocolRep, Repository repositoryStorage, int port) {
        this.ffe = ffe;
        this.repository = repository;
        this.protocolRep = protocolRep;
        this.repositoryStorage = repositoryStorage;
        this.server_port = port;
        this.server_run = false;
        repository.setServerListener(this);
    }

    /**
     * Initializes the server and starts it on the previously selected port.
     *
     * @see Thread#start()
     * @see ClientHandler
     * @since 1.0
     */
    @Override
    public void run() {
        try {
            // Allow looping in the loop and create a socket server
            SSLServerSocketFactory sslserversocketfactory = (SSLServerSocketFactory) SSLServerSocketFactory.getDefault();
            this.server = sslserversocketfactory.createServerSocket(this.server_port);
            this.server_run = true;
            while (this.server_run) {
                // Accepting connection requests (blocking)
                SSLSocket client = (SSLSocket) this.server.accept();

                // Create a new Handler client by passing these dependencies to it
                ClientHandler clientHandler = new ClientHandler(this.repository, client, ffe, protocolRep, new Context(repositoryStorage, repository));
                // Add the client handler to its repository (clienthandlerrepository)
                this.repository.addClient(clientHandler);
                // Start the thread
                clientHandler.start();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Stops the server and terminates the new connection.
     *
     * @since 1.0
     */
    public void stop() {
        if (this.server_run) {
            try {
                this.server_run = false;
                this.server.close();
            } catch (IOException ignored) {
            }
        }
    }

}
