package lightcontainer.interfaces;

import lightcontainer.domains.Task;
import lightcontainer.domains.client.StoreProcessor;
import lightcontainer.domains.server.MulticastServerListener;
import lightcontainer.repository.StoreProcessorRepository;

import java.time.LocalDateTime;
import java.util.Collection;

/**
 * A communication interface between a {@link StoreProcessor} and the {@link StoreProcessorRepository}.
 */
public interface MulticastSPR {
    /**
     * Setter, allow to define the ServerListener of a repository.
     *
     * @param server ServerListener to set as default.
     */
    void setServerListener(MulticastServerListener server);

    /**
     * Add a StorePorcessor.
     *
     * @param store Store processor to add.
     */
    boolean addStore(StoreProcessor store);

    String findDomain();

    void assignTask(String stor, Task task);

    boolean hasDomain(String domain);

    int domainCount();

    /**
     * Déconnecte tous les SBE
     */
    void disconnectDomains();

    /**
     * Permet de déconnecter un SBE
     *
     * @param domain Le domaine du SBE à déconnecter
     */
    void closeStore(String domain);

    /**
     * Permet de mettre à jours la dernière annonce de ce SBE au FFE
     * @param domain Le domain s'annoncant
     */
    void updateLastAnnounce(String domain);


    /**
     * Permet de récupérer les noms des domaines connectés au FFE
     * @return Les noms des domaines connectés au FFE
     */
    Collection<String> getDomains();

    /**
     * Permet de récupérer la dernière annonce d'un SBE
     * @return La dernière annonce d'un SBE
     */
    LocalDateTime getLastAnnounce(String domain);
}
