package lightcontainer.protocol.rules.reader;

import lightcontainer.domains.client.Context;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.protocol.ProtocolReader;
import lightcontainer.protocol.rules.writer.FilesRule;

import java.util.List;

import static lightcontainer.protocol.StandardizedDefinitions.CLIENT_FILE_LIST;

/**
 * Règle permettant de récupérer la liste des fichiers d'un utilisateur
 */
public class FilelistRule extends ProtocolReader {
    // Constants
    private static final String PATTERN = CLIENT_FILE_LIST;

    private static final String NAME = "FILELIST";

    private final ProtocolRepository protocolRep;

    // Constructor
    public FilelistRule(ProtocolRepository protocolRep) {
        super(NAME, PATTERN);
        this.protocolRep = protocolRep;
    }

    public class Result extends ProtocolResult {
        public Result(Context context) {
            super(context);
        }
    }

    /**
     * Cette méthode est appelée lors de l'exécution de la règle
     *
     * @param data Paramètres pour créer la commande.
     */
    @Override
    protected FilelistRule.Result onExecuted(Context context, String... data) {
        FilelistRule.Result result = new Result(context);
        List<String> files = context.getStringifiedFilesOf();
        if (files != null) {
            result.setResultCommand(this.protocolRep.executeWriter(context, FilesRule.NAME, files.toArray(new String[0])), ResultCmdReceiver.CLIENT);
        } else {
            result.setResultCommand(this.protocolRep.executeWriter(context, FilesRule.NAME), ResultCmdReceiver.CLIENT);
        }
        return result;
    }

    @Override//TODO ???
    protected <T extends ProtocolResult> T onError(Context context) {
        return super.onError(context);
    }
}
