package lightcontainer.protocol.rules.reader;

import lightcontainer.domains.client.Context;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.protocol.ProtocolReader;
import lightcontainer.protocol.rules.writer.GetFileErrorRule;
import lightcontainer.protocol.rules.writer.RetrieveFileRule;
import lightcontainer.storage.ReadOnlyFile;

import java.util.Iterator;

import static lightcontainer.protocol.StandardizedDefinitions.CLIENT_GET_FILE;

public class GetFileRule extends ProtocolReader {
    // Constants
    //"^GETFILE ([^ !]{1,20})\r\n$"
    private static final String PATTERN = CLIENT_GET_FILE;
    private static final String NAME = "GETFILE";
    // -- arguments
    private static final int FILE_NAME = 0; // Index file name.

    // Variables
    private final ProtocolRepository protocolRep;

    // Constructor
    public GetFileRule(ProtocolRepository protocolRep) {
        super(NAME, PATTERN);
        this.protocolRep = protocolRep;
    }

    /**
     * Rule Result
     */
    public class Result extends ProtocolResult {
        // Variables
        private final String fileName;

        // Constructor
        public Result(Context context, String fileName) {
            super(context);
            this.fileName = fileName;
        }
    }

    /**
     * Cette méthode est appelée lors de l'exécution de la règle
     *
     * @param context Utilisateur context.
     * @param data    Paramètres pour créer la commande.
     */
    @Override
    protected GetFileRule.Result onExecuted(Context context, String... data) {
        GetFileRule.Result result = new GetFileRule.Result(context, data[FILE_NAME]);

        ReadOnlyFile file = context.getFile(data[FILE_NAME]);

        if (file != null) {
            // Précision du store back end demandé pour traiter cette commande.
            String requestDomain = extractRequestDomain(file.getStorageIterator());
            result.setRequestDomain(requestDomain);

            // Save into bundle
            context.putDataString("fileName", file.getName());
            context.putDataLong("fileSize", file.getSize());
            context.putDataString("fileIV", file.getIv());

            // Create cmd for storebacked
            result.setResultCommand(protocolRep.executeWriter(context, RetrieveFileRule.NAME, context.getHashedFileName(file.getName())), ResultCmdReceiver.STOREBACKEND);
        } else {
            result.setResultCommand(protocolRep.executeWriter(context, GetFileErrorRule.NAME), ResultCmdReceiver.CLIENT);
        }
        return result;
    }

    /**
     * Cette méthode est appelée lors d'une erreur de la règle
     */
    protected ProtocolReader.ProtocolResult onError(Context context) {
        ProtocolReader.ProtocolResult result = new ProtocolReader.ProtocolResult(context);
        // Make getFileErrorRule
        result.setResultCommand(protocolRep.executeWriter(context, GetFileErrorRule.NAME), ResultCmdReceiver.CLIENT);
        return result;
    }

    /**
     * TMP
     * TODO : But futur est de pouvoir en avoir plusieurs (sbe)
     * Cette méthode permet de choisir le domaine voulu.
     *
     * @param storageIterator Les domaines
     * @return Le domain choisi
     */
    private String extractRequestDomain(Iterator<String> storageIterator) {
        return storageIterator.next();
    }
}
