package lightcontainer.protocol.rules.reader;

import lightcontainer.domains.client.Context;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.protocol.ProtocolReader;
import lightcontainer.protocol.rules.writer.EraseFileRule;
import lightcontainer.protocol.rules.writer.RemoveFileErrorRule;
import lightcontainer.storage.ReadOnlyFile;

import java.util.Iterator;

import static lightcontainer.protocol.StandardizedDefinitions.CLIENT_REMOVE_FILE;

public class RemoveFileRule extends ProtocolReader {
    //"^REMOVEFILE ([^ !]{1,20})\r\n$"
    private static final String PATTERN = CLIENT_REMOVE_FILE;

    private static final String NAME = "REMOVEFILE";

    private static final int FILENAME = 0;


    private final ProtocolRepository protocolRep;

    public RemoveFileRule(ProtocolRepository protocolRep) {
        super(NAME, PATTERN);
        this.protocolRep = protocolRep;
    }

    public class Result extends ProtocolResult {
        private final String fileName;

        public Result(Context context, String fileName) {
            super(context);
            this.fileName = fileName;
        }

        public String getFileName() {
            return fileName;
        }
    }

    @Override
    protected RemoveFileRule.Result onExecuted(Context context, String... data) {
        RemoveFileRule.Result result = new RemoveFileRule.Result(context, data[FILENAME]);
        ReadOnlyFile file = context.getFile(result.getFileName());

        if (file != null) {
            String extendedFileName = context.getHashedFileName(result.getFileName());
            context.putDataString("fileName", result.getFileName());
            result.setResultCommand(protocolRep.executeWriter(context, EraseFileRule.NAME, extendedFileName), ResultCmdReceiver.STOREBACKEND);

            // Précision du store back end demandé pour traiter cette commande.
            String requestDomain = extractRequestDomain(file.getStorageIterator());
            result.setRequestDomain(requestDomain);
        } else {
            result.setResultCommand(protocolRep.executeWriter(context, RemoveFileErrorRule.NAME), ResultCmdReceiver.CLIENT);
        }

        return result;
    }

    @Override
    protected ProtocolReader.ProtocolResult onError(Context context) {
        ProtocolReader.ProtocolResult result = new ProtocolReader.ProtocolResult(context);
        result.setResultCommand(protocolRep.executeWriter(context, RemoveFileErrorRule.NAME), ResultCmdReceiver.CLIENT);
        return result;
    }

    /**
     * TODO : But futur est de pouvoir en avoir plusieurs
     * Cette méthode permet de choisir le domaine voulu.
     *
     * @param storageIterator Les domaines
     * @return Le domain choisi
     */
    private String extractRequestDomain(Iterator<String> storageIterator) {
        return storageIterator.next();
    }
}
