package lightcontainer.protocol.rules.reader;

import lightcontainer.domains.client.Context;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.protocol.ProtocolReader;
import lightcontainer.protocol.rules.writer.SignErrorRule;
import lightcontainer.protocol.rules.writer.SignOkRule;

import static lightcontainer.protocol.StandardizedDefinitions.CLIENT_SIGN_IN;

/**
 * Règle permettant de gérer la connection d'un utilisateur
 */
public class SigninRule extends ProtocolReader {
    // Constants
    //"^SIGNIN ([A-Za-z0-9]{5,20}) ([^ !]{5,50})\r\n$"
    private static final String PATTERN = CLIENT_SIGN_IN;

    private static final String NAME = "SIGNIN";

    private static final int LOGIN = 0;  // Index du domain dans le tableau de données
    private static final int PASSWORD = 1;  // Index du port dans le tableau de données

    private final ProtocolRepository protocolRep;

    // Constructor
    public SigninRule(ProtocolRepository protocolRep) {
        super(NAME, PATTERN);
        this.protocolRep = protocolRep;
    }

    public class Result extends ProtocolResult {
        // Variables
        private final String login;
        private final String password;

        // Result constructor
        public Result(Context context, String login, String password) {
            super(context);
            this.login = login;
            this.password = password;
        }

        public String getLogin() {
            return login;
        }

        public String getPassword() {
            return password;
        }

    }

    @Override
    protected SigninRule.Result onExecuted(Context context, String... data) {
        SigninRule.Result result = new SigninRule.Result(context, data[LOGIN], data[PASSWORD]);

        if (context.signIn(result.getLogin(), result.getPassword())) {
            result.setResultCommand(this.protocolRep.executeWriter(context, SignOkRule.NAME), ResultCmdReceiver.CLIENT);
        } else {
            result.setResultCommand(this.protocolRep.executeWriter(context, SignErrorRule.NAME), ResultCmdReceiver.CLIENT);
        }

        return result;
    }

    @Override
    protected ProtocolReader.ProtocolResult onError(Context context) {
        ProtocolReader.ProtocolResult result = new ProtocolReader.ProtocolResult(context);

        result.setResultCommand(protocolRep.executeWriter(context, SignErrorRule.NAME), ResultCmdReceiver.CLIENT);
        return result;
    }
}
