package lightcontainer.protocol.rules.writer;

import lightcontainer.domains.client.Context;
import lightcontainer.protocol.ProtocolWriter;
import lightcontainer.utils.DeepFileEraser;

import static lightcontainer.protocol.StandardizedDefinitions.FFE_GET_FILE_RESULT_ERROR;

/**
 * Règle utilisé dans le cas ou une erreur survient lors
 * de la demande de récupération d'un fichier.
 */
public class GetFileErrorRule extends ProtocolWriter {
    // Constants
    //"^GETFILE_ERROR\r\n$"
    private static final String PATTERN = FFE_GET_FILE_RESULT_ERROR;
    public static final String NAME = "GETFILE_ERROR";

    // Variables
    private final String storagePath;

    // Constructors
    public GetFileErrorRule(String storagePath) {
        super(NAME, PATTERN);
        this.storagePath = storagePath;
    }

    @Override
    protected ProtocolResult onExecuted(Context context, String... data) {
        // Delete the file
        try {
            DeepFileEraser.eraseFile(String.format("%s/%s", storagePath, context.getDataString("hashedFileName")));
        } catch (DeepFileEraser.DeepFileEraserException ignore) { }
        // Return new protocol result
        return new ProtocolResult(context);
    }
}
