package lightcontainer.protocol.rules.writer;

import lightcontainer.domains.client.Context;
import lightcontainer.protocol.ProtocolWriter;
import lightcontainer.utils.DeepFileEraser;
import lightcontainer.utils.FileSender;

import java.io.IOException;
import java.io.OutputStream;

import static lightcontainer.protocol.StandardizedDefinitions.FFE_GET_FILE_RESULT_OK;

public class GetFileOkRule extends ProtocolWriter {
    // Constants
    //"^GETFILE_OK ([^ !]{1,20}) ([0-9]{1,10})\r\n$"
    private static final String PATTERN = FFE_GET_FILE_RESULT_OK;
    public static final String NAME = "GETFILE_OK";
    // -- params
    private static final int FILE_NAME = 0;  // Index file name hashed
    private static final int FILE_SIZE = 1;  // Index file size

    // Variables
    private final String storagePath;

    // Constructors
    public GetFileOkRule(String storagePath) {
        super(NAME, PATTERN);
        this.storagePath = storagePath;
    }

    public class Result extends ProtocolWriter.ProtocolResult {
        // Variables
        private final String filename;
        private final long filesize;

        // Constructors
        public Result(Context context, String filename, long filesize) {
            super(context);
            this.filename = filename;
            this.filesize = filesize;
        }

        /**
         * Oh yeahh baby by tonton EndMove ;-)
         *
         * @param writer Buffer à remplir qui sera envoyer via le réseau
         */
        @Override
        public void write(OutputStream writer) throws IOException {
            FileSender fileSender = new FileSender(storagePath);
            fileSender.sendFile(
                    getContext().getDataString("hashedFileName"),
                    writer,
                    getContext().getDataLong("encryptedFileSize"), // Encrypted file size (because data is parsing into AES system)
                    getContext().getAesKey(),
                    getContext().getDataString("fileIV")
            );

            try {
                DeepFileEraser.eraseFile(String.format("%s/%s", storagePath, getContext().getDataString("hashedFileName")));
            } catch (DeepFileEraser.DeepFileEraserException ignore) { }
        }
    }

    @Override
    protected GetFileOkRule.Result onExecuted(Context context, String... data) {
        return new GetFileOkRule.Result(context, data[FILE_NAME], Integer.parseInt(data[FILE_SIZE]));
    }
}
