package lightcontainer.protocol.rules.writer;

import lightcontainer.domains.client.Context;
import lightcontainer.protocol.ProtocolWriter;
import lightcontainer.utils.DeepFileEraser;
import lightcontainer.utils.ShaHasher;

import static lightcontainer.protocol.StandardizedDefinitions.FFE_SAVE_FILE_ERROR;

/**
 * Règle signifiant que la sauvegarde d'un fichier a échoué
 */
public class SaveFileErrorRule extends ProtocolWriter {
    //"^SAVEFILE_ERROR\r\n$"
    private static final String PATTERN = FFE_SAVE_FILE_ERROR;

    public static final String NAME = "SAVEFILE_ERROR";

    private final String storagePath;

    public SaveFileErrorRule(String storagePath) {
        super(NAME, PATTERN);
        this.storagePath = storagePath;
    }


    @Override
    protected ProtocolResult onExecuted(Context context, String... data) {
        ProtocolResult result = new ProtocolResult(context);
        String fileName = context.getDataString("fileName");
        String fileNameSalt = context.getDataString("fileNameSalt");

        if (fileName != null && fileNameSalt != null) {
            // Suppression du fichier temporaire dans le stockage du FFE
            ShaHasher hasher = new ShaHasher(context.getLogin() + "_" + context.getDataString("fileName"));
            try {
                DeepFileEraser.eraseFile(String.format("%s/%s", this.storagePath,
                        hasher.fromSalt(hasher.saltToByte(context.getDataString("fileNameSalt")))));
            } catch (DeepFileEraser.DeepFileEraserException ignore) {
            }
        }
        return result;
    }
}
