package lightcontainer.protocol.rules.writer;

import lightcontainer.domains.client.Context;
import lightcontainer.protocol.ProtocolWriter;
import lightcontainer.utils.DeepFileEraser;

import static lightcontainer.protocol.StandardizedDefinitions.FFE_SAVE_FILE_OK;

/**
 * Règle signifiant que la sauvegarde d'un fichier fût un succès
 */
public class SaveFileOkRule extends ProtocolWriter {
    //"^SAVEFILE_OK\r\n$"
    private static final String PATTERN = FFE_SAVE_FILE_OK;

    public static final String NAME = "SAVEFILE_OK";

    private final String storagePath;

    public SaveFileOkRule(String storagePath) {
        super(NAME, PATTERN);
        this.storagePath = storagePath;
    }

    @Override
    protected ProtocolWriter.ProtocolResult onExecuted(Context context, String... data) {
        ProtocolWriter.ProtocolResult result = new ProtocolWriter.ProtocolResult(context);

        // Sauvegarder dans JSON
        context.addFile(context.getDataString("fileName"), context.getDataString("fileNameSalt"), context.getDataLong("size"), context.getDataString("iv"), context.getDomain());

        // Suppression du fichier temporaire dans le stockage du FFE
        String hashedFileName = context.getHashedFileName(context.getDataString("fileName"));

        try {
            DeepFileEraser.eraseFile(String.format("%s/%s", storagePath, hashedFileName));
        } catch (DeepFileEraser.DeepFileEraserException ignore) { }
        return result;
    }
}
