package lightcontainer.protocol.rules.writer;

import lightcontainer.domains.client.Context;
import lightcontainer.protocol.ProtocolWriter;
import lightcontainer.utils.FileSender;

import java.io.IOException;
import java.io.OutputStream;

import static lightcontainer.protocol.StandardizedDefinitions.FFE_SENDFILE;

/**
 * Règle envoyée au SBE, demandant la sauvegarde d'un fichier.
 */
public class SendfileRule extends ProtocolWriter {
    //"^SENDFILE [A-Za-z0-9]{50,200} [0-9]{1,10} [A-Za-z0-9]{50,200}\r\n$"
    private static final String PATTERN = FFE_SENDFILE;

    public static final String NAME = "SENDFILE";

    private static final int HASHED_FILE_NAME = 0; // Index file name hashed.
    private static final int FILE_SIZE = 1; // Index file size.
    private static final int HASHED_FILE_CONTENT = 2; // Index file content hashed.

    private final String storagePath;

    public SendfileRule(String storagePath) {
        super(NAME, PATTERN);
        this.storagePath = storagePath;
    }

    public class Result extends ProtocolWriter.ProtocolResult {

        private final String hashedFileName;
        private final long fileSize;
        private final String hashedFileContent;

        public Result(Context context, String hashedFileName, long fileSize, String hashedFileContent) {
            super(context);
            this.hashedFileName = hashedFileName;
            this.fileSize = fileSize;
            this.hashedFileContent = hashedFileContent;
        }

        @Override
        public void write(OutputStream writer) throws IOException {
            super.write(writer);

            FileSender fileSender = new FileSender(storagePath);
            fileSender.sendFile(hashedFileName, writer);
        }
    }

    @Override
    protected SendfileRule.Result onExecuted(Context context, String... data) {
        return new SendfileRule.Result(context, data[HASHED_FILE_NAME], Integer.parseInt(data[FILE_SIZE]), data[HASHED_FILE_CONTENT]);
    }
}
