package lightcontainer.repository;

import lightcontainer.domains.client.ClientHandler;
import lightcontainer.domains.server.UnicastServerListener;
import lightcontainer.interfaces.UnicastCHR;
import lightcontainer.protocol.ProtocolWriter;

import java.util.ArrayList;
import java.util.List;
// TODO : C'est genre un ClientHandlerManager quoi hein, normal qu'il fasse blinder de chose ;)

/**
 * ClientHandlerRepository
 * <p>
 * Repository storing ClientHandler class.
 * Contains some utility functions.
 *
 * @author Jérémi NIHART <j.nihart@student.helmo.be>
 * @version 1.0
 * @see ClientHandler
 * @see AutoCloseable
 * @since 1.0
 */
public class ClientHandlerRepository implements AutoCloseable, UnicastCHR {
    // Variable
    private final List<ClientHandler> handlers;
    private UnicastServerListener server;

    // Constructor
    public ClientHandlerRepository() {
        this.handlers = new ArrayList<>();
    }

    /**
     * Setter, allow to define the ServerListener of a repository.
     * & Start the server.
     *
     * @param server ServerListener to set as default.
     * @since 1.0
     */
    @Override
    public void setServerListener(UnicastServerListener server) {
        this.server = server;
        new Thread(server).start();
    }

    /**
     * Add a ClientHandler.
     *
     * @param client Client Handler to add.
     * @since 1.0
     */
    @Override
    public void addClient(ClientHandler client) {
        this.handlers.add(client);
    }

    /**
     * Permet de savoir si un client est déjà connecté avec ce login.
     *
     * @param login Login du client.
     */
    @Override
    public boolean hasClient(String login) {
        for (ClientHandler client : handlers) {
            if (client.getLogin() != null && client.getLogin().equals(login)) {
                return true;
            }
        }
        return false;
    }


    @Override
    public void disconnect(ClientHandler client) {
        if (handlers.remove(client))
            client.close();
    }

    @Override
    public void respondToClient(String login, ProtocolWriter.ProtocolResult response) {
        for (ClientHandler client : handlers) {
            if (client.getLogin().equals(login)) {
                client.respond(response);
                break;
            }
        }
    }

    /**
     * AutoClosable Function
     * Closes all ClientHandlers stored in this repository and deallocates all resources.
     *
     * @since 1.0
     */
    @Override
    public void close() {
        // Stop the server.
        this.server.stop();
        // Stop each clients.
        this.handlers.forEach(ClientHandler::close);
    }
}
