package lightcontainer.repository;

import lightcontainer.domains.Task;
import lightcontainer.domains.client.Context;
import lightcontainer.domains.client.StoreProcessor;
import lightcontainer.interfaces.ClientHandlerFFE;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.interfaces.StoreProcessorFFE;
import lightcontainer.protocol.ProtocolWriter;

import java.util.Deque;
import java.util.Iterator;
import java.util.concurrent.ConcurrentLinkedDeque;

public class FileFrontEnd implements ClientHandlerFFE, StoreProcessorFFE {
    // Variables
    private final Deque<Task> tasks = new ConcurrentLinkedDeque<>();
    private final ClientHandlerRepository clientRepository; // TODO -> pourquoi pas une interface ? end
    private final StoreProcessorRepository storeRepository; // TODO -> pourquoi pas une interface ? end
    private ProtocolRepository protocolRepository;

    // Constructor
    public FileFrontEnd(ClientHandlerRepository clientRepo, StoreProcessorRepository storeRepo, ProtocolRepository protocolRepository) {
        this.clientRepository = clientRepo;
        this.storeRepository = storeRepo;
        this.protocolRepository = protocolRepository;
    }

    /**
     * Appelé quand nouvelle tâche
     */
    public void alertStoreProcessors(Task task) {
        // On avertit les stor processors d'une nouvelle tâche
        String stor = storeRepository.findDomain();
        if (stor != null) {
            storeRepository.assignTask(stor, task);
            task.setDomain(stor);
        }
    }

    /**
     * Permet à un {@link StoreProcessor} d'avertir le FFE qu'il est disponible
     *
     * @param storeDomain Le SBE qui s'est occupé de la tâche
     * @param response La réponse à envoyer au client
     */
    @Override
    public void onStoreAvailable(String storeDomain, ProtocolWriter.ProtocolResult response) {
        responseToClient(storeDomain, response);

        Iterator<Task> it = tasks.iterator();
        while (it.hasNext()) {
            Task task = it.next();
            if (task.getDomain() == null || task.getDomain().equals(storeDomain)) { // Si tâche sans domaine ou si domaine requis pour la tâche est le domaine
                task.setDomain(storeDomain);
                storeRepository.assignTask(storeDomain, task);
                return;
            }
        }
    }

    @Override
    public void onStoreDisconnect(String domain) {
        responseToClient(domain, null);
        Iterator<Task> it = tasks.iterator();
        while (it.hasNext()) {
            Task task = it.next();
            if (storeRepository.domainCount() == 0 || (task.getDomain() != null && task.getDomain().equals(domain))) {
                clientRepository.respondToClient(it.next().getClient(), null);
                it.remove(); // Suppression de la tâche
            }
        }
        this.storeRepository.closeStore(domain);
    }

    private void responseToClient(String storeDomain, ProtocolWriter.ProtocolResult response) {
        Iterator<Task> it = tasks.iterator();
        while (it.hasNext()) {
            Task task = it.next();
            if (task.isResponseOfClient(storeDomain)) {
                clientRepository.respondToClient(task.getClient(), response);
                it.remove(); // Suppression de la tâche
                break;
            }
        }
    }


    @Override
    public void newCommand(Context context, ProtocolWriter.ProtocolResult command, String requestDomain) {
        Task task = Task.newInstance(context, command, requestDomain);
        tasks.add(task);
        alertStoreProcessors(task);
    }

    @Override
    public boolean canExecuteCommand(String domain) {
        return domain == null ? storeRepository.domainCount() > 0 : storeRepository.hasDomain(domain);
    }
}
