package lightcontainer.storage;

import lightcontainer.utils.BCryptHasher;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * User represents a user of the system.
 *
 * @author Maximilien LEDOUX <m.ledoux@student.helmo.be>
 * @version 1.0
 * @since 1.0
 */
public class User {
    // Variables
    private final String name;
    private final String password;
    private final String aesKey;
    private final Map<String, File> files;

    // Constructor
    public User(String name, String password, String aesKey, Map<String, File> files) {
        this.name = name;
        this.password = password;
        this.aesKey = aesKey;
        this.files = files;
    }

    public String getName() {
        return name;
    }

    public String getPassword() {
        return password;
    }

    public String getAesKey() {
        return aesKey;
    }

    public Iterator<File> fileIterator() {
        return files.values().iterator();
    }

    public List<String> getFilesWithSize() {
        Iterator<File> fileIterator = fileIterator();
        List<String> result = new ArrayList<>();
        while (fileIterator.hasNext()) {
            File file = fileIterator.next();
            String stringify = file.getName() + "!" + file.getSize();
            result.add(stringify);
        }
        return result;
    }

    public File getFile(String fileName) {
        return this.files.get(fileName);
    }

    /**
     * @param file The file to add.
     * @return False if a file with the same name already exists. Otherwise, adds the file and returns true.
     */
    public boolean addFile(File file) {
        if (canAddFile()) {
            this.files.put(file.getName(), file);
            return true;
        }
        return false;
    }

    public boolean canAddFile() {
        return this.files.size() <= 50;
    }

    /**
     * @param fileName The name of the file to delete.
     * @return True if the file was deleted. False otherwise.
     */
    public boolean deleteFile(String fileName) {
        if (this.files.containsKey(fileName)) {
            this.files.remove(fileName);
            return true;
        } else {
            return false;
        }
    }

    /**
     * @param file    The file that needs a storage
     * @param storage The storage name
     * @return True if the storage was added to the file. False otherwise.
     */
    public boolean addStorage(File file, String storage) {
        if (this.files.containsKey(file.getName())) {
            return file.addStorage(storage);
        } else {
            return false;
        }
    }

    /**
     * @param password The plain text password to test
     * @return True if password matches user's hashed password. False otherwise
     */
    public boolean verifyPassword(String password) {
        return BCryptHasher.checkPassword(password, this.password);
    }
}
